import { CancellationToken, ContributionProvider, DisposableCollection } from '@theia/core';
import { PreferenceService } from '@theia/core/lib/browser';
import { interfaces } from '@theia/core/shared/inversify';
import { IBufferRange, ILink, ILinkDecorations } from 'xterm';
import { TerminalWidget } from './base/terminal-widget';
import { TerminalContribution } from './terminal-contribution';
import { LinkContext } from './terminal-link-helpers';
import { TerminalWidgetImpl } from './terminal-widget-impl';
export declare const TerminalLinkProvider: unique symbol;
export interface TerminalLinkProvider {
    provideLinks(line: string, terminal: TerminalWidget, cancelationToken?: CancellationToken): Promise<TerminalLink[]>;
}
export declare const TerminalLink: unique symbol;
export interface TerminalLink {
    startIndex: number;
    length: number;
    tooltip?: string;
    handle(): Promise<void>;
}
export declare const XtermLink: unique symbol;
export declare const XtermLinkFactory: unique symbol;
export declare type XtermLinkFactory = (link: TerminalLink, terminal: TerminalWidgetImpl, context: LinkContext) => ILink;
export declare function createXtermLinkFactory(ctx: interfaces.Context): XtermLinkFactory;
export declare class TerminalLinkProviderContribution implements TerminalContribution {
    protected readonly terminalLinkContributionProvider: ContributionProvider<TerminalLinkProvider>;
    protected readonly xtermLinkFactory: XtermLinkFactory;
    onCreate(terminalWidget: TerminalWidgetImpl): void;
    protected provideTerminalLinks(terminal: TerminalWidgetImpl, line: number, provideLinks: (links?: ILink[]) => void): Promise<void>;
}
export declare class XtermLinkAdapter implements ILink {
    text: string;
    range: IBufferRange;
    decorations: ILinkDecorations;
    protected link: TerminalLink;
    protected terminalWidget: TerminalWidgetImpl;
    protected context: LinkContext;
    protected readonly preferences: PreferenceService;
    protected toDispose: DisposableCollection;
    protected mouseEnteredHover: boolean;
    protected mouseLeftHover: boolean;
    initializeLinkFields(): void;
    hover(event: MouseEvent, text: string): void;
    protected scheduleHover(event: MouseEvent): void;
    protected showHover(event: MouseEvent): void;
    leave(event: MouseEvent): void;
    protected cancelHover(): void;
    activate(event: MouseEvent, text: string): void;
    protected executeLinkHandler(): void;
    protected isModifierKeyDown(event: MouseEvent | KeyboardEvent): boolean;
    protected wasTouchEvent(event: MouseEvent, lastTouchEnd?: TouchEvent): boolean;
}
//# sourceMappingURL=terminal-link-provider.d.ts.map